from py2neo import Graph

class Neo4j():
    connection = None
    settings = {
        'host': '',
        'username': '',
        'password': ''
    }

    @staticmethod
    def initialize(host, username, password):
        Neo4j.settings['host'] = host
        Neo4j.settings['username'] = username
        Neo4j.settings['password'] = password

    @staticmethod
    def graph(): 
        if Neo4j.connection is None:
            print("Opening Database Connection")
            Neo4j.connection = Graph(
                host=Neo4j.settings['host'], 
                user=Neo4j.settings['username'], 
                password=Neo4j.settings['password']
            )

        return Neo4j.connection


from pandas import DataFrame
from neo4jdecorators.db import Neo4j

# decorator to inject transaction into a class method
def Transactional(method):
    def openTransaction(self=None, tx=None):
        # call method with transaction
        transaction = tx or Neo4j.graph().begin()
        result = None

        try:
            result = method(self=self, tx = transaction)
            
            transaction.commit()
        except:
            transaction.rollback()
            raise

        return result

    return openTransaction 

# decorator to run query over a given or injected transaction
def Query(statement, asTable=False):
    def neo4jQuery(method):
        def startTxAndQuery(self, tx=None, parameters=None):
            if tx is None:
                tx = Neo4j.graph()

            result = tx.run(statement, parameters=parameters)

            if asTable:
                result = DataFrame(result.data())

            return method(self, result=result)

        return startTxAndQuery

    return neo4jQuery



#!/usr/bin/env python3

from distutils.core import setup

setup(
    name='neo4j-decorators',
    version='1.2',
    description='Decorators do use py2neo library like Spring Data annotations',
    long_description=open('README.md').read(),
    author='Arlindo Neto',
    author_email='<arlindosilvaneto@gmail.com>',
    maintainer='Arlindo Neto',
    maintainer_email='<arlindosilvaneto@gmail.com>',
    url='',
    packages=['neo4jdecorators'],
    install_requires=[
        'py2neo',
        'pandas'
    ],
    classifiers=[
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', 'Operating System :: OS Independent'
    ]
)
